//-----------------------------------------------------------------------------
// Title         : Amber system
// Project       : SHMAC
//-----------------------------------------------------------------------------
// File          : amber_system_<acc module>.v
// Author        : Asbjørn Djupdal  <djupdal@idi.ntnu.no>, edited by Marton Teilgård
//				   for module generation
// Created       : 06.09.2013
// Last modified : ??.06.2014
//-----------------------------------------------------------------------------
// Description : Wishbone based system containing CPU, timer,
// interrupt controller and tile register
// 
//-----------------------------------------------------------------------------
// Copyright (c) 2013 by ARM/CARD
//------------------------------------------------------------------------------
// Modification history :
// ??.07.2013 : created
//-----------------------------------------------------------------------------

`include "common_defs.v"

#0
  #(
    parameter tile_x = 4'b0,
    parameter tile_y = 4'b0,
    parameter cpu_id = 8'hff
	)
   (
    input wire          i_clk,
    input wire          i_rst,

    input wire          i_irq,

    input wire          i_system_rdy,

    output wire [31:0]  o_wb_adr,
    output wire [15:0]  o_wb_sel,
    output wire         o_wb_we,
    input wire [127:0]  i_wb_dat,
    output wire [127:0] o_wb_dat,
    output wire         o_wb_cyc,
    output wire          o_wb_stb,
    input wire          i_wb_ack,
    input wire          i_wb_err,
    output wire         o_ld_excl

    );

   ////////////////////////////////////////////////////////////////////////////
	
   
   wire                 irq;
   wire                 firq;
	
   wire [31:0] 			in0_o_wbm_adr;
   wire [15:0] 			in0_o_wbm_sel;
   wire 				in0_o_wbm_we;
   wire [127:0] 		in0_o_wbm_dat;
   wire					in0_o_wbm_cyc;
   reg					in0_o_wbm_stb;
   wire [127:0] 		in0_i_wbm_dat;
   
	
   wire [31:0] 			acc_o_wbm_adr;
   wire [15:0] 			acc_o_wbm_sel;
   wire 				acc_o_wbm_we;
   wire [127:0] 		acc_o_wbm_dat;
   wire					acc_o_wbm_cyc;
   wire					acc_o_wbm_stb;
   wire [127:0] 		acc_i_wbm_dat;
   
	
   reg [127:0]          wb_dat_r_cpu;
   wire [127:0]         wb_dat_r_tileregs;
   wire [127:0]         wb_dat_r_timer;
   wire [127:0]         wb_dat_r_irq;
   wire [127:0]         wb_dat_r_acc; 

   reg                  wb_ack_cpu;
   wire                 wb_ack_tileregs;
   wire                 wb_ack_timer;
   wire                 wb_ack_irq;
   wire                 wb_ack_acc;	

   reg                  wb_err_cpu;
   wire                 wb_err_tileregs;
   wire                 wb_err_timer;
   wire                 wb_err_irq;
   wire                 wb_err_acc; 
	
   wire                 wb_stb_cpu;
   reg                  wb_stb_tileregs;
   reg                  wb_stb_timer;
   reg                  wb_stb_irq;
   reg                  wb_stb_acc; 

   wire [2:0]           irq_timers;
   wire					irq_acc;
   wire [31:1]          int_sources;

   wire [31:0]          tile_base = `TILE_BASE;
   wire [15:0]          tilereg = `TILE_REGS;
   wire [15:0]          timer_mod = `TIMER;
   wire [15:0]          int_ctrl = `INT_CTRL;
   wire [15:0]			acc = `ACC; 

   ////////////////////////////////////////////////////////////////////////////

   a25_core u_amber
     (
      .i_clk          (i_clk),
      .i_rst          (i_rst),
      
      .i_irq          (irq),
      .i_firq         (firq),

      .i_system_rdy   (i_system_rdy),
      
      .o_wb_adr       (in0_o_wbm_adr),
      .o_wb_sel       (in0_o_wbm_sel),
      .o_wb_we        (in0_o_wbm_we),
      .i_wb_dat       (wb_dat_r_cpu),
      .o_wb_dat       (in0_o_wbm_dat),
      .o_wb_cyc       (in0_o_wbm_cyc),
      .o_wb_stb       (wb_stb_cpu),
      .i_wb_ack       (wb_ack_cpu),
      .i_wb_err       (wb_err_cpu),
      .o_ld_excl      (o_ld_excl)
      );

   tile_regs
     #(
       .WB_DWIDTH (128),
       .WB_SWIDTH (16),
       .tile_x (tile_x),
       .tile_y (tile_y),
       .cpu_id (cpu_id)
       )
   u_tile_regs
     (
      .i_clk (i_clk),
      .i_rst (i_rst),

      .i_wb_adr (in0_o_wbm_adr),
      .i_wb_sel (in0_o_wbm_sel),
      .i_wb_we (in0_o_wbm_we),
      .o_wb_dat (wb_dat_r_tileregs),
      .i_wb_dat (in0_o_wbm_dat),
      .i_wb_cyc (in0_o_wbm_cyc),
      .i_wb_stb (wb_stb_tileregs),
      .o_wb_ack (wb_ack_tileregs),
      .o_wb_err (wb_err_tileregs)
      );

   timer_module 
     #(
       .WB_DWIDTH (128),
       .WB_SWIDTH (16)
       )
   u_timer
     (
      .i_clk (i_clk),
      .i_rst (i_rst),

      .i_wb_adr (in0_o_wbm_adr),
      .i_wb_sel (in0_o_wbm_sel),
      .i_wb_we (in0_o_wbm_we),
      .o_wb_dat (wb_dat_r_timer),
      .i_wb_dat (in0_o_wbm_dat),
      .i_wb_cyc (in0_o_wbm_cyc),
      .i_wb_stb (wb_stb_timer),
      .o_wb_ack (wb_ack_timer),
      .o_wb_err (wb_err_timer),
      .o_timer_int (irq_timers)
      );

   interrupt_controller  
     #(
       .WB_DWIDTH (128),
       .WB_SWIDTH (16)
       )
   u_irq_ctrl
     (
      .i_clk (i_clk),
      .i_rst (i_rst),

      .i_wb_adr (in0_o_wbm_adr),
      .i_wb_sel (in0_o_wbm_sel),
      .i_wb_we (in0_o_wbm_we),
      .o_wb_dat (wb_dat_r_irq),
      .i_wb_dat (in0_o_wbm_dat),
      .i_wb_cyc (in0_o_wbm_cyc),
      .i_wb_stb (wb_stb_irq),
      .o_wb_ack (wb_ack_irq),
      .o_wb_err (wb_err_irq),
      
      .o_irq (irq),
      .o_firq (firq),
      
      .i_int_sources (int_sources)
      );
	
	#1
	(
		.i_clk(i_clk), 
		.i_rst(i_rst), 
		.o_irq(irq_acc), 
		.o_wbm_adr(acc_o_wbm_adr), 
		.o_wbm_sel(acc_o_wbm_sel), 
		.o_wbm_we(acc_o_wbm_we), 
		.i_wbm_dat(acc_i_wbm_dat), 
		.o_wbm_dat(acc_o_wbm_dat), 
		.o_wbm_cyc(acc_o_wbm_cyc), 
		.o_wbm_stb(acc_o_wbm_stb), 
		.i_wbm_ack(acc_i_wbm_ack), 
		.i_wbm_err(acc_i_wbm_err), 
		.i_wbs_adr (in0_o_wbm_adr),
      .i_wbs_sel (in0_o_wbm_sel),
      .i_wbs_we (in0_o_wbm_we),
      .o_wbs_dat (wb_dat_r_acc),
      .i_wbs_dat (in0_o_wbm_dat),
      .i_wbs_cyc (in0_o_wbm_cyc),
      .i_wbs_stb (wb_stb_acc),
      .o_wbs_ack (wb_ack_acc),
      .o_wbs_err (wb_err_acc)
	);
	
	wb_arbiter u_arbiter (
		.i_rst(i_rst), 
		.o_wbm_adr(o_wb_adr), 
		.o_wbm_sel(o_wb_sel), 
		.o_wbm_we(o_wb_we), 
		.i_wbm_dat(i_wb_dat), 
		.o_wbm_dat(o_wb_dat), 
		.o_wbm_cyc(o_wb_cyc), 
		.o_wbm_stb(o_wb_stb), 
		.i_wbm_ack(i_wb_ack), 
		.i_wbm_err(i_wb_err), 
		.in0_o_wbm_adr(in0_o_wbm_adr),
		.in0_o_wbm_sel(in0_o_wbm_sel), 
		.in0_o_wbm_we(in0_o_wbm_we), 
		.in0_i_wbm_dat(in0_i_wbm_dat), 
		.in0_o_wbm_dat(in0_o_wbm_dat), 
		.in0_o_wbm_cyc(in0_o_wbm_cyc), 
		.in0_o_wbm_stb(in0_o_wbm_stb), 
		.in0_i_wbm_ack(in0_i_wbm_ack), 
		.in0_i_wbm_err(in0_i_wbm_err), 
		.in1_o_wbm_adr(acc_o_wbm_adr), 
		.in1_o_wbm_sel(acc_o_wbm_sel), 
		.in1_o_wbm_we(acc_o_wbm_we), 
		.in1_i_wbm_dat(acc_i_wbm_dat), 
		.in1_o_wbm_dat(acc_o_wbm_dat), 
		.in1_o_wbm_cyc(acc_o_wbm_cyc), 
		.in1_o_wbm_stb(acc_o_wbm_stb), 
		.in1_i_wbm_ack(acc_i_wbm_ack), 
		.in1_i_wbm_err(acc_i_wbm_err)
	);
	
   ////////////////////////////////////////////////////////////////////////////

   assign int_sources = {26'b0,irq_acc, irq_timers, i_irq};

   always @* begin
      // default is router
      wb_dat_r_cpu = in0_i_wbm_dat;
      wb_ack_cpu = in0_i_wbm_ack;
      wb_err_cpu = in0_i_wbm_err;
      in0_o_wbm_stb = wb_stb_cpu;
      wb_stb_tileregs = 0;
      wb_stb_timer = 0;
      wb_stb_irq = 0;
	  wb_stb_acc = 0;

      // override default for local wishbone addresses
      if(in0_o_wbm_adr[31:16] == tile_base[31:16]) begin
         case(in0_o_wbm_adr[15:12])
           tilereg[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_tileregs;
              wb_ack_cpu = wb_ack_tileregs;
              wb_err_cpu = wb_err_tileregs;
              in0_o_wbm_stb = 0;
              wb_stb_tileregs = wb_stb_cpu;
           end

           timer_mod[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_timer;
              wb_ack_cpu = wb_ack_timer;
              wb_err_cpu = wb_err_timer;
              in0_o_wbm_stb = 0;
              wb_stb_timer = wb_stb_cpu;
           end

           int_ctrl[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_irq;
              wb_ack_cpu = wb_ack_irq;
              wb_err_cpu = wb_err_irq;
              in0_o_wbm_stb = 0;
              wb_stb_irq = wb_stb_cpu;
           end
			  
			  acc[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_acc;
              wb_ack_cpu = wb_ack_acc;
              wb_err_cpu = wb_err_acc;
              in0_o_wbm_stb = 0;
              wb_stb_acc = wb_stb_cpu;
           end
         endcase
      end
        
   end

endmodule
